/* -*-c-*-
 *
 * Matches anagrams and subgrams
 *
 * (c) 2001 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of Anag: a simple wordgame helper.
 *
 * Anag is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Anag is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Anag; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Header files ------------------------------------------------------*/

#include "anag.h"

/*----- Data structures ---------------------------------------------------*/

typedef struct node_gram {
  node n;
  size_t sz;
  unsigned f[UCHAR_MAX + 1];
} node_gram;

/*----- Main code ---------------------------------------------------------*/

/* --- @compile@ --- *
 *
 * Arguments:	@node_gram *n@ = pointer to node to fill in
 *		@const char *p@ = pointer to string to compile from
 *
 * Returns:	The length of the string.
 *
 * Use:		Fills in an anagram node with letter frequencies.
 */

static size_t compile(node_gram *n, const char *p)
{
  size_t len = 0;
  unsigned i;
  memset(n->f, 0, sizeof(n->f));
  while (*p) {
    i = (unsigned char)*p++;
    n->f[i]++;
    len++;
  }
  return (len);
}

/* --- Node matcher --- */

static int n_gram(node *nn, const char *p, size_t sz)
{
  node_gram *n = (node_gram *)nn;
  unsigned f[UCHAR_MAX];
  const char *q;
  unsigned i;

  if (n->sz && sz != n->sz)
    return (0);
  memcpy(f, n->f, sizeof(f));
  q = p + sz;
  while (p < q) {
    i = (unsigned char)*p++;
    if (f[i])
      f[i]--;
    else if (f['.'])
      f['.']--;
    else
      return (0);
  }
  return (1);
}

/* --- Node creation --- */

node *anagram(const char *const *av)
{
  node_gram *n = xmalloc(sizeof(*n));
  n->n.func = n_gram;
  n->sz = compile(n, av[0]);
  return (&n->n);
}

node *subgram(const char *const *av)
{
  node_gram *n = xmalloc(sizeof(*n));
  n->n.func = n_gram;
  n->sz = 0;
  compile(n, av[0]);
  return (&n->n);
}

/*----- That's all, folks -------------------------------------------------*/
